/* File:      chain.h
   Date:      5/05/03
   Author:    C. Paciorek
   Purpose:   header file for chain.C
*/

class dataset;
class spatial_parm;
class nonspatial_parm;
class sparse_cov_mat_nonstat;
class covariate_info;

/* Global variables */
extern dataset* data;  /* data is global because chain and parms need access */
extern covariate_info* covariates;
extern int prior;
extern int* idum;

/* 
The chain classes initialize and hold the parameters, step through the Markov chain, and store and report the results.

Currently the chain class has the same correlation structure for all the GPs used in defining the kernel structure

*/


class chain{
/* chain with all GP priors for the kernel parameters sharing the same correlation structure */
public:

  chain(string log_fn_in,string eval_fn_in,string fun_form_fn_in,string dat_fn_in,string init_fn_in,string final_fn_in,string parm_fn_in,string out_fn_in,int num_its_in,int storage_interval_in,int storage_space_in);   

  ~chain(void);

  void store(int it_cur,int store,int burnin);    
  /* stores results of each iteration */

  void do_a_step(int it_cur,int burnin);
  /* does a round/iteration of MCMC, updating each parameter */

  void report(int it_cur,int status);  
  /* reports final results */

private:
  /* file names */
  string log_fn;
  string eval_fn;
  string out_fn;             /* base name of iterate output files */
  string final_fn;           /* file of final parameter values */

  /* storage for parameter iterates */
  mat_pp<double> *storage1;  
  mat_pp<double> *storage2;  
  mat_pp<double> *storage3;  
  mat_pp<double> *storage4;  
  mat_pp<double> *storage5;  
  mat_pp<double> *storage6;  
  int storage_space;         /* size of block for storing iterates */
  int storage_interval;      /* interval at which iterates are saved */
  /* so if storage_space=1000, storage_interval=10, then every 10th is saved and and after 10000 iterates, with 1000 saved, the output is written to file */
  ofstream output;

  /* basic information variables */
  int num_its;
  int it_run;                /* number of iteration in current block */
  int block;                 /* current printing block (output is printed out in blocks of iterates */
  int num_datapoints;        /* num_testpoints plus num_trainpoints */
  int num_testpoints;
  int num_trainpoints;
  int num_covars;
  int num_eigenvecs;         /* number of eigenvector coordinates */
  int num_angles;            /* is a function of num_covars */

  /* parameters in model  */
  spatial_parm* kernel;           /* prior covariance structure */
  spatial_parm* fun;              /* regression function */
  spatial_parm** eigenval;        /* kernel eigenvalues */
  spatial_parm** eigenvec;        /* kernel eigenvector coordinate values */

  nonspatial_parm* eta;         /* residual sd */

  /* covariance structures */
  sparse_cov_mat_nonstat* cov;       /* training set */
  sparse_cov_mat_nonstat* cov_full;  /* train plus test set */

  /* vectors of pointers to kernel eigenstructures -- these are passed into the cov_mat variables */
  double** eigenval_dbl;        /* training set */
  double** eigenvec_dbl;        /* training set */
  double** eigenval_full_dbl;   /* train plus test set */  
  double** eigenvec_full_dbl;   /* train plus test set */


  /* initial values and ongoing storage space for current parameters */
  double* fun_omega_init;
  double** eigenval_omega_init;
  double** eigenvec_omega_init;
};






