/* File:      corr.C
   Date:      5/19/03  (originally written fall 2001)
   Author:    C. Paciorek
   Purpose:   basic correlation functions; includes Gneiting and Das functions
   Compile:   g++ -c -Wall
   Usage:     
   Arguments: 
   Requires:  see just below
   Reference: Stein, 1999, p. 50 discusses this parameterization of the Matern; rkbesl function for modified Bessel function of second kind taken from www.netlib.org specfun collection */

#include "base.h"
#include "corr.h"

double matern_corr_euc(double euc_distance,double kappa,double nu){
 

  double outrkb;

  // 2*sqrt(nu) is so that as nu->\infty, correlation fxn approaches squared exponential; with nu=1/2; this is equivalent to exp(-2*sqrt(nu)*dist/kappa)
  double dx=2.0*sqrt(nu)*euc_distance/kappa;
  
  if(dx>500){   // rkbesl underflows for values larger than about 700
    return(0.0);
  }
  if((dx<1e-8 && nu>=15.0) || (dx<1e-20 && nu<15.0)){ // rkbesl overflow depends on nu
    return(1.0); 
  }
  // results get pretty flaky for small dx, so not sure I would trust even the cutoffs above, hence the line below checking for corr > 1.0

  int nb=1+(int)floor(nu);
  double alpha=(double)nu-nb+1;
  int ize=1;
  int ncalc;
  double* tmp=new double[nb];
  rkbesld_(&dx,&alpha,&nb,&ize,tmp,&ncalc);  // modified Bessel function of second kind - returns values for several orders in tmp
  outrkb=tmp[nb-1];  // K_nu(dx) -- use value of order nu
  delete [] tmp;

  double outval=exp(-lgamma(nu)-(nu-1.0)*log(2.0)+nu*log(dx)+log(outrkb));
  // calculate Matern correlation
  if(outval>1.0){
    cerr << setprecision(16) << "Warning; correlation exceeds 1.0 " << outval << " " << dx << " " << nu << " " << euc_distance << endl;
    outval=1.0;
  }
  return(outval);  
}


