/* File:      covariates.h
   Date:      03/24/02
   Author:    C. Paciorek
   Purpose:   header file for covariates.C
*/

class sparse_cov_mat;
class sparse_cov_mat_stat;
class sparse_cov_mat_nonstat;
class dataset;


const int NO_PERMUTE=0;
const int PERMUTE=1;

/* structure and methods for covariates */
class covariate_info {
public:
  covariate_info(string covar_fn_in,string perm_fn_in,int num_trainpoints_in,int num_testpoints_in,int num_covars_in);
  ~covariate_info(void);
  void fill_adj(void);      /* determines which covariates are neighbors (within range s.t. may have non-zero correlation, and fills adjacency structures for non-discrete version */
  void calc_datadata_distance(int x,int y,double *dist,int permute); /* calcs Euclidean distance between covariates x and y */
  void choose_adjinfo(int size);  /* choose between adjacency information for training set and that for combined training and test set for use in Cholesky stuff in cov_mat.C */
  void scale_covars(void);  /* scales covarate values to (0,1) independently for each covariate */
  int num_covars;
  int num_eigenvecs;        /* number of eigenvector coordinates involved in nonstationary correlation structure */
  int num_angles;           /* number of Givens angles involved in stationary correlation structure */
  int num_datapoints;       /* training points plus test points */
  int num_trainpoints;  
  int num_testpoints;  
  friend dataset;           /* needs access to permutation to permute data */
  friend sparse_cov_mat;    /* needs access to adj,perm etc. for doing symbolic factorization */

private:
  mat_pp<double>* covars;    /* covariates for each observation */ 

  /* objects used in sparse error covariance structure */

  /* adjacency information */
  int *xadj;                                 /* for each covariate, pointer to start of neighbors in adjncy - Fortran style indexing */   
  int *adjncy;                               /* adjacency vector in George & Liu form - Fortran style indexing */
  int adjncy_len;
  int *xadj_train;                                 /* for each covariate, pointer to start of neighbors in adjncy - Fortran style indexing */   
  int *adjncy_train;                               /* adjacency vector in George & Liu form - Fortran style indexing */
  int adjncy_len_train;
  int *xadj_full;                                 /* for each covariate, pointer to start of neighbors in adjncy - Fortran style indexing */   
  int *adjncy_full;                               /* adjacency vector in George & Liu form - Fortran style indexing */
  int adjncy_len_full;

  /* permutation objects */
  int* perm;  /* permutation of covariates for evaluation runs in which different permutations are used because chain results can differ depending on permutation of covariates:  perm = {2,3,4,1,5} means that first covariate is the orig 2nd one, 2nd is the original 3rd one, etc. */
  int* invp;  /* inverse of perm */

  
  double* covar_min;   /* max and min values used for scaling covariates to (0,1) */
  double* covar_max;

};


