/* File:      density.C
   Date:      5/19/03  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   evaluates various log-density functions 
   Compile:   g++ -c -Wall -ansi
   Usage:     N/A
   Arguments: N/A
   Requires:  see just below
   Reference: written by C. Paciorek; see Anderson et al. 1987 for cos_dens
*/


#include "base.h"
#include "density.h"

// beta(1,3) density on the interval (low,up) rescaled to (0,1) 
double beta_dens(double arg,double low,double up){
  if(arg<=up && arg>=low){
    arg=(arg-low)/(up-low);
    return(2.0*log(arg));
  }
  else{
    return(NEG_INFTY);
  }
}

// density for Givens angles given in Anderson et al. 1987
double cos_dens(double arg,double order,double garb){
  if(arg<(-PI) || arg>PI){
    return(NEG_INFTY);
  }
  else{
    return(order*log(fabs(cos(arg))));
  }
  return(0.0);
}

// Gamma, parameterization with mean=alpha*beta
double gamma_dens(double arg,double alpha,double beta)
{
  if(arg<0 || alpha<=0 || beta <=0) {return(DENSITY_ERROR);}
  return((alpha-1)*log(arg)-arg/beta);
}

// Univariate Normal
double normal_dens(double arg,double mean, double sd)
{
  if(sd <=0) {return(DENSITY_ERROR);}
  return(-0.5*(arg-mean)*(arg-mean)/(sd*sd));
}

// Multivariate Normal
double mvn_dens(int size,double* values,double sd){
  double sum=0.0; int i;
  if(sd <=0) {return(DENSITY_ERROR);}
  for(i=0;i<size;i++){
    sum+=values[i]*values[i];
  }
  return(-0.5*sum/(sd*sd));
}

double uniform_dens(double arg,double low,double up){
  if(arg<=up && arg>=low){
    return(0.0);
  }
  else{
    return(NEG_INFTY);
  }
}

 





