/* File:      permtools.C
   Date:      5/19/03  (originally written fall 2001)
   Author:    C. Paciorek
   Purpose:   permutes and inverse permutes a vector
   Compile:   g++ -c -Wall -ansi
   Usage:     N/A
   Arguments: N/A
   Requires:  see just below
   Reference: modified from lib/spars.ext taken off web 
*/
#include "base.h"
#include "permtools.h"

/* perm = {2,3,4,1,5} means that first element of the permuted vector is the orig 2nd one, 2nd is the original 3rd one, etc. */

/*****************************************************************************/
/* 'invrse' computes the inverse permutation of the permuation vector 'perm',*/
/* i.e. if 1,2,...,n are mapped onto perm[1],perm[2],...,perm[n], then       */
/* invp[perm[1]],invp[perm[2],...,invp[perm[n]] are mapped back to 1,2,...,n */
/* In C, 'perm, invp' have SUBSCRIPTS 0,1,2,...,*n-1, while the components of*/
/* 'perm','invp' have CONTENTS between 1 and *n.                             */
/*****************************************************************************/
void invrse(int *n, int *perm, int *invp,int permstart){
   int   i;
   for (i=0; i<*n; i++)
       invp[perm[i]-permstart]=i+permstart;
}


/*****************************************************************************/
/* 'permrv' maps rhs[1],...,rhs[n] onto rhs[perm[1]],...,rhs[perm[n]].       */
/* In C, 'rhs, perm' have SUBSCRIPTS 0,1,2,...,*n-1, while the components of */
/* 'perm' are between 1 and *n.                                              */
/*****************************************************************************/

// note that one can use invp as argument perm to perform the inverse
// this template not working for some reason with double
template<class T>
void permrv(int *n,T *rhs, int *perm, int permstart)
{
   int    i;
   T *dummy;
   
   dummy=new T[*n];

   for (i=0; i<*n; i++)
     dummy[i]=rhs[perm[i]-permstart];
   for (i=0; i<*n; i++)
     rhs[i]=dummy[i];
   delete [] dummy;
}

// hard-coded version for double 
void permrv(int *n, double *rhs, int *perm, int permstart)
{
   int    i;
   double *dummy;
   
   dummy=new double[*n];

   for (i=0; i<*n; i++)
     dummy[i]=rhs[perm[i]-permstart];
   for (i=0; i<*n; i++)
     rhs[i]=dummy[i];
   delete [] dummy;
}








